/*
Floricu Andreea-Ana-Maria --- PROIECT PM
*/
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>

#include "spi.h"
#include "usart.h"
#include "ST7735R_TFT.h"
#include "ds18b20.h"

#define SOIL_MIN 30 // umiditatea optima minima
#define SOIL_MAX 70 // umididatea optima maxima

#define TEMP_MAX 40 // temperatura maxima optima
#define TEMP_MIN  15 // temperatura minima optima

#define EPSILON 21

/*din lab*/
void ADC_init()
{
	ADMUX = (1 << REFS0);
    ADCSRA = (1 << ADEN) | (5 << ADPS0);
}
 /*functie cu care iau datele de la senzorul de umiditate sol*/
int ADC_Read()
{
	ADMUX = 0x40;
	ADCSRA |= (1<<ADSC);
	while ((ADCSRA &(1<<ADIF))==0);
	ADCSRA |=(1<<ADIF);
	return(ADCW);
}

int main(){

    SPI_init();
    ST7735R_Begin();
	ADC_init();
	USART0_init(); 
	
	
	DDRD |= (1 << PD7);

	int valoare_masurata = 0;
	double procent_umiditate = 0.0;
    char buf[100];
    char t_buf[10];
    t_buf[0] = ' ';
    t_buf[3] = ':';
    t_buf[6] = ':';
    t_buf[9] = 0;
    int valoare_precedenta = 0;
    int timp = 0;
    // timer = 0 => nu afisez timpul
    int timer = 0;
    int secunde, minute, ore;
	int8_t digit = 1;
	uint16_t decimal = 1;
	char mod;
	int timp_real = 0;
	int comenzi_bluetooth = 0;
	
	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 222, 91, 174);
	
	ST7735R_DrawText(0, 50, "     SISTEM DE  " , 174, 20, 20, 222, 91, 174);
	ST7735R_DrawText(0, 70, "    MONITORIZARE" , 174, 20, 20, 222, 91, 174);
	ST7735R_DrawText(0, 90, "    A PLANTELOR" , 174, 20, 20, 222, 91, 174);

schimba:
	mod = USART0_receive(); // aleg modul de functionare
	
	if (mod == 'r') { // modul ales este cel de afisare in timp real pe LCD
		timp_real = 1;
		comenzi_bluetooth = 0;
	} else if (mod == 'b') { // modul ales este cel de comenzi personalizate prin bluetooth
		timp_real = 0;
		comenzi_bluetooth = 1;
	}
	
	// modul de functionare timp_real
    while(timp_real == 1) {
	

		// iau datele de la senzori
		valoare_masurata = ADC_Read();
		therm_read_temperature(&digit, &decimal);

        // planta este in curs de udare (umiditatea creste cu cel putin 2%)
        if (valoare_precedenta - valoare_masurata > EPSILON) {
            // reset timer
            timer = 0; // in timpul udarii nu se afiseaza contorul
            timp = 0; //resetez contorul dupa ce planta a fost udata
        }

        // udarea s-a terminat
        if (valoare_precedenta <= valoare_masurata) {
            //start timer
            timer = 1;
        }

        timp++;
		
		// fundal LCD 
		ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 222, 91, 174);

		// obtin un procent de umiditate al solului
		procent_umiditate = (double)(1023 - valoare_masurata) * 50 / valoare_masurata;

		ST7735R_DrawText(0, 10, "     SISTEM DE  " , 174, 20, 20,  222, 91, 174);
		ST7735R_DrawText(0, 25, "    MONITORIZARE" , 174, 20, 20, 222, 91, 174);
		ST7735R_DrawText(0, 40, "    A PLANTELOR" , 174, 20, 20,  222, 91, 174);

		sprintf(buf, " Umiditate: %d%%", (int)procent_umiditate);
		ST7735R_DrawText(0, 60, buf, 0, 255, 0, 222, 91, 174);
		sprintf(buf, " Temperatura: %d,%d C", (int)digit, (int)decimal);
		ST7735R_DrawText(0, 80, buf, 255, 0, 0, 222, 91, 174);
		
        if (procent_umiditate > SOIL_MAX) {
            PORTD |= (1 << PD7);
        }

        else if (procent_umiditate > SOIL_MIN) {
            PORTD ^= (1 << PD7);
        }

        else {
            PORTD &= ~(1 << PD7);
        }

        if (timer == 1) {
            secunde = timp % 60;
            minute = (timp / 60) % 60;
            ore = timp / 3600;
            t_buf[1] = ore / 10 + '0';
            t_buf[2] = ore % 10 + '0';
            t_buf[4] = minute / 10 + '0';
            t_buf[5] = minute % 10 + '0';
            t_buf[7] = secunde / 10 + '0';
            t_buf[8] = secunde % 10 + '0';
			
			ST7735R_DrawText(0, 110, " Timp scurs de la", 0, 0, 255, 222, 91, 174);
			ST7735R_DrawText(0, 120, "   ultima udare:", 0, 0, 255, 222, 91, 174);
			ST7735R_DrawText(0, 140, t_buf, 0, 0, 0, 222, 91, 174);
        }

        valoare_precedenta = valoare_masurata;
		
        _delay_ms(1000);	
		
    }
	
	if(comenzi_bluetooth == 1) {
		char t[30];
		
		//USART0_print("Monitorizare de la distanta.\r\n");
		sprintf(t, "Monitorizare de la distanta.\r\n");
		USART0_print(t);
		
		while (1) {
			char com;
			char text[30];
			char buf[30];
			
			// iau datele de la senzori
			valoare_masurata = ADC_Read();
			therm_read_temperature(&digit, &decimal);
			procent_umiditate = (double)(1023 - valoare_masurata) * 50 / valoare_masurata;
			
			// primesc comanda prin bluetooth
			com = USART0_receive();
			

			if (com == 't') {
				sprintf(text, "Temperatura = %d,%d C\r\n", (int)digit, (int)decimal);
				sprintf(buf, "Temperatura = %d,%d C", (int)digit, (int)decimal);

			    USART0_print(text);
				ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 255, 0);

				ST7735R_DrawText(0, 80, buf, 0, 0, 255, 0, 255, 0);
				_delay_ms(10);
				
				if (digit < TEMP_MAX && digit > TEMP_MIN) {
					sprintf(text, "Temperatura este propice pentru udat :) \r\n");
					USART0_print(text);
					_delay_ms(10);
				} else {
					sprintf(text, "Temperatura NU este propice pentru udat :) \r\n");
					USART0_print(text);
					_delay_ms(10);
				}
				
			} else if (com == 'u') {
				sprintf(text, "Umiditate: %d%%\r\n", (int)procent_umiditate);
				sprintf(buf, " Umiditate: %d%%", (int)procent_umiditate);

			    USART0_print(text);
				ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 255, 0);

				ST7735R_DrawText(0, 80, buf, 0, 0, 255, 0, 255, 0);
				_delay_ms(10);

				// pamantul nu este suficientde ud --> planta trebuie udata
				if (procent_umiditate < SOIL_MIN) {
					sprintf(text, "Planta trebuie udata !!!\r\n");
					USART0_print(text);
					_delay_ms(10);
			    } else  if (procent_umiditate > SOIL_MAX){
					sprintf(text, "Planta are prea multa apa !!!\r\n");
					USART0_print(text);
					_delay_ms(10);
				} else if (procent_umiditate > SOIL_MIN && procent_umiditate < SOIL_MAX) {
					sprintf(text, "Planta este in parametrii optimi de udare :) \r\n");
					USART0_print(text);
					_delay_ms(10);
				}
			} else if (com == 'm') {
				sprintf(text, "Introduceti modul de functionare dorit:\r\n");
				USART0_print(text);
				goto schimba;
			}
			else if (com == 's') {
				sprintf(text, "MONITORIZARE DE LA DISTANTA FINALIZATA !\r\n");
				USART0_print(text);
				break;
			}
		}
		_delay_ms(1000);
	}
    return 0;
}
